package java.time.format;

import java.util.concurrent.ConcurrentHashMap;
import java.util.Objects;
import java.util.Collections;
import java.util.HashSet;
import java.text.DecimalFormatSymbols;
import java.util.Set;
import java.util.Locale;
import java.util.concurrent.ConcurrentMap;

public final class DecimalStyle
{
    public static final DecimalStyle STANDARD;
    private static final ConcurrentMap<Locale, DecimalStyle> CACHE;
    private final char zeroDigit;
    private final char positiveSign;
    private final char negativeSign;
    private final char decimalSeparator;
    
    public static Set<Locale> getAvailableLocales() {
        Locale[] availableLocales = DecimalFormatSymbols.getAvailableLocales();
        HashSet<Object> c = new HashSet(availableLocales.length);
        Collections.addAll(c, availableLocales);
        return c;
    }
    
    public static DecimalStyle ofDefaultLocale() {
        return of(Locale.getDefault(Locale.Category.FORMAT));
    }
    
    public static DecimalStyle of(Locale obj) {
        Objects.requireNonNull(obj, "locale");
        DecimalStyle decimalStyle = (DecimalStyle)DecimalStyle.CACHE.get(obj);
        if (decimalStyle == null) {
            DecimalStyle.CACHE.putIfAbsent(obj, create(obj));
            decimalStyle = (DecimalStyle)DecimalStyle.CACHE.get(obj);
            return decimalStyle;
        }
        return decimalStyle;
    }
    
    private static DecimalStyle create(Locale locale) {
        DecimalFormatSymbols instance = DecimalFormatSymbols.getInstance(locale);
        int zeroDigit = instance.getZeroDigit();
        int n = '+';
        int minusSign = instance.getMinusSign();
        int decimalSeparator = instance.getDecimalSeparator();
        if (zeroDigit == '0' && minusSign == '-' && decimalSeparator == '.') {
            return DecimalStyle.STANDARD;
        }
        return new DecimalStyle((char)zeroDigit, (char)n, (char)minusSign, (char)decimalSeparator);
    }
    
    private DecimalStyle(char zeroDigit, char positiveSign, char negativeSign, char decimalSeparator) {
        this.zeroDigit = zeroDigit;
        this.positiveSign = positiveSign;
        this.negativeSign = negativeSign;
        this.decimalSeparator = decimalSeparator;
    }
    
    public char getZeroDigit() {
        return this.zeroDigit;
    }
    
    public DecimalStyle withZeroDigit(char c) {
        if (c == this.zeroDigit) {
            return this;
        }
        return new DecimalStyle(c, this.positiveSign, this.negativeSign, this.decimalSeparator);
    }
    
    public char getPositiveSign() {
        return this.positiveSign;
    }
    
    public DecimalStyle withPositiveSign(char c) {
        if (c == this.positiveSign) {
            return this;
        }
        return new DecimalStyle(this.zeroDigit, c, this.negativeSign, this.decimalSeparator);
    }
    
    public char getNegativeSign() {
        return this.negativeSign;
    }
    
    public DecimalStyle withNegativeSign(char c) {
        if (c == this.negativeSign) {
            return this;
        }
        return new DecimalStyle(this.zeroDigit, this.positiveSign, c, this.decimalSeparator);
    }
    
    public char getDecimalSeparator() {
        return this.decimalSeparator;
    }
    
    public DecimalStyle withDecimalSeparator(char c) {
        if (c == this.decimalSeparator) {
            return this;
        }
        return new DecimalStyle(this.zeroDigit, this.positiveSign, this.negativeSign, c);
    }
    
    int convertToDigit(char c) {
        int n = c - this.zeroDigit;
        return (n >= 0 && n <= 9) ? n : -1;
    }
    
    String convertNumberToI18N(String s) {
        if (this.zeroDigit == '0') {
            return s;
        }
        int n = this.zeroDigit - 48;
        char[] charArray = s.toCharArray();
        for (int i = 0; i < charArray.length; ++i) {
            charArray[i] += n;
        }
        return new String(charArray);
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if ((o instanceof DecimalStyle)) {
            DecimalStyle decimalStyle = (DecimalStyle)o;
            return this.zeroDigit == decimalStyle.zeroDigit && this.positiveSign == decimalStyle.positiveSign && this.negativeSign == decimalStyle.negativeSign && this.decimalSeparator == decimalStyle.decimalSeparator;
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        return this.zeroDigit + this.positiveSign + this.negativeSign + this.decimalSeparator;
    }
    
    @Override
    public String toString() {
        return "DecimalStyle[" + this.zeroDigit + this.positiveSign + this.negativeSign + this.decimalSeparator + "]";
    }
    
    static {
        STANDARD = new DecimalStyle('0', '+', '-', '.');
        CACHE = new ConcurrentHashMap(16, 0.75f, 2);
    }
}
